/*===========================================================================*

  Prog    : RDVECT
            Lee tabla de vectores del floppy generado con WRBOOT.

  Compile : bcc -ml rdvect.c

/*===========================================================================*/

#define  VERSION "1.0"         /* version */

#include <dos.h>
#include <bios.h>
#include <alloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dir.h>
#include <mem.h>
#include <conio.h>   
#include <fcntl.h>
#include <sys\stat.h>

/*===========================================================================*
/* define's */
/*===========================================================================*/

#define RESETDSK        0       /* function reset disk       */
#define GETSTATUS       1       /* function get drive status */
#define READVBLK        2       /* function read block       */
#define WRITEVBLK       3       /* function write disk block */
#define VERIFYTRK       4       /* function verify disk addr */
#define BLOCK           512     /* buffer size for block     */
#define VECLEN          2048    /* vector table size         */
#define VECTOR_FILE     "VECTOR.BIN"  /* vector image file   */

/*===========================================================================*
/* variables */
/*===========================================================================*/

static int debug;                        /* debug flag */
static unsigned char vecbuf[VECLEN];     /* vector buffer */

/*===========================================================================*
 *                              prterr                                       * 
 *===========================================================================*/
prterr(int byt)       /* print biosdisk error */
{
      fprintf(stderr,"rdvect: ");
      
      if ((byt & 0x80) != 0)
	 fprintf(stderr, "Disk timed-out.\n");
      else if ((byt & 0x40) != 0)
	 fprintf(stderr, "Seek failure.\n");
      else if ((byt & 0x20) != 0)
	 fprintf(stderr, "Controller error.\n");
      else if ((byt & 0x10) != 0)
	 fprintf(stderr, "Data error on disk read (CRC).\n");
      else if ((byt & 0x08) != 0)
	 fprintf(stderr, "DMA overrun on operation.\n");
      else if ((byt & 0x04) != 0)
	 fprintf(stderr, "Request sector not found.\n");
      else if ((byt & 0x02) != 0)
	 fprintf(stderr, "Disk write protected.\n");
      else if ((byt & 0x01) != 0)
	 fprintf(stderr, "Ilegal command passed to driver.\n");
      else printf("Status = 0x%x \n", byt);

   return(0);
}

/*===========================================================================*
 *                              main                                         * 
 *===========================================================================*/
void main()
{
    FILE *fd;
    int byt;

    printf("\nVector Floppy Reader V%s - All rights reserved\n", VERSION);
    printf("1995 - C.M.T. Software Utilities Inc.\n");

    printf("rdvect: reading vector information from floppy ...\n");

    byt = biosdisk(READVBLK, 0, 0, 0, 2, 4, vecbuf);

    if (byt != 0)  /* trato de hacerlo 2 veces si hay error */
        byt = biosdisk(READVBLK, 0, 0, 0, 2, 4, vecbuf);

   /* si hubo error imprimo mensaje en pantalla */

    if (byt != 0) {
       prterr(byt);     /* print error */
       exit(1);
       }

    printf("rdvect: writing vector information to vector file ...\n");

    if ((fd = fopen(VECTOR_FILE,"wb")) == NULL) {
       perror("fopen vector file");
       exit(errno);
       }

    if (fwrite(vecbuf,sizeof(char),VECLEN,fd) != VECLEN) {
       perror("fwrite vector file");
       exit(errno);
       }

    (void)fclose(fd);       /* close vector file */

    printf("rdvect: done.\n");

}

