	name    bblurt
	title   'bblurt.com'

; Boot sector program to print bios interrupt jump table 
; and bios data area to screen. Each nybble is encoded as
; a hex digit

cseg    segment para    public  'CODE'
	org 0h
	assume  cs:cseg,ds:cseg,es:cseg,ss:cseg

bblock  proc near

BOOTCODE        =       7c00h           ; boot code entry
BOOTSEG         =       7c0h            ; boot segment
VECTOR          =       00h             ; vector segment
SCREEN1         =       960             ; number of 2 byte vector entries per 
					;  full screen (80x24)
SCREEN2         =       960
SCREEN3         =       128

;               Bootstrap Entrypoint
;

bb1:
	mov     ax, cs          ; resets segments registers
	mov     es, ax
	mov     ss, ax
	mov     ax, 7c0h        ; data segment
	mov     ds, ax
	mov     sp,7c00h        ; set up a stack
    
	mov     ax,0            ; loads vector area buffer
	mov     es,ax
	mov     si,ax
	mov     bx,3            ; number of screens
	mov     cx,SCREEN1      ; counter for first screen full of data
	cld                     ; arrange for autoincrement
	xor     ah,ah           ; we only use al below
     
readloop:       
	push ds                 ; ds needs to be set to 0 to access vector
	mov ax,0                ; area as data, then reset to this segment
	mov ds,ax
	lodsb                   ; get next vector byte in al
	pop ds
	mov dl,al               ; save it
	shr al,1                ; shift to get at high nybble
	shr al,1
	shr al,1
	shr al,1
	mov di,ax                
	mov al,byte ptr hexdgs[di]
	call printchar          ; print indexed hex digit
	mov al,dl               ; remember vector byte stored in dl
	and al,0FH              ; mask out high nybble
	mov di,ax
	mov al,byte ptr hexdgs[di]
	call printchar
	dec cx                  ; decrement counter
	jz done1
	jmp readloop            ; fetch next vector byte
done1:
	call kbhit              ; wait for user to hit a key
	dec bx
	cmp bx,2
	jb done2
	mov cx,SCREEN2          ; prepare to present 2nd screen of data
	jmp readloop
done2:
	cmp bx,1
	jb done
	mov cx,SCREEN3           ; prepare to present 3rd screen of data
				 ; print blank line for visibility
	mov al,0DH               ; Carriage return
	call printchar
	mov al,0AH               ; Line feed
	call printchar
	jmp readloop 
done:
	jmp done                 ; loops forever, but that's ok

printchar:
	push ax                 ; al has char to print
	push bx                                                         
	mov  ah, 0EH            ; int 10h function to print char
	mov  bh,0               ; active page
	int 10H
	pop bx
	pop ax
	ret

kbhit:                          ; Wait for user to strike a key
	push ax
	mov ah, 0
	int 16H
	pop ax
	ret


bblock  endp

hexdgs:  db    '0123456789ABCDEF'

cseg    ends

end bblock


