	name    bblurt2
	title   'bblurt2.com'

; Boot sector program to print bios interrupt jump table 
; and bios data area to screen. Each nybble is encoded as
; a hex digit. In this version they are presented to the user in 8 digit
; chunks, for example
; 012:230AFF0F
; where 012 is the line (=interrupt) number
; and the rest is 2 word (hi byte low byte) segment offset pair

; Contains some of Claudio Tantignone's code from version that stored
; jump table on a floppy disk. Thanks, Claudio!

cseg    segment para    public  'CODE'
	org 0h
	assume  cs:cseg,ds:cseg,es:cseg,ss:cseg

bblock  proc near

BOOTCODE        =       7c00h           ; boot code entry
BOOTSEG         =       7c0h            ; boot segment
VECTOR          =       00h             ; vector segment
CHUNKSIZE       =       4               ; number of double hex-chars 
CHUNKS          =       512             ; number of chunks in table

;               Bootstrap Entrypoint
;

bb1:
	mov     ax, cs          ; resets segments registers
	mov     es, ax
	mov     ss, ax
	mov     ax, 7c0h        ; data segment
	mov     ds, ax
	mov     sp,7c00h        ; set up a stack
    
	mov     ax,0            ; loads vector area buffer
	mov     es,ax
	mov     si,ax
	mov     cx,CHUNKS       ; counter for how many chunks
	push    cx              ; store it
	cld                     ; arrange for autoincrement
	xor     ah,ah           ; we only use al below
	mov bx, offset banner
	call print              ; print our banner
	mov bx, offset prompt
	call print              ; inform user
	call linenumber         ; initial linenumber
	mov cx, CHUNKSIZE       ; prepare to print first chunk
	jmp chunkloop

chunksloop:     
	call kbhit              ; wait while user looks at screen
	mov cx,CHUNKSIZE*2+4    ; prepare to clear last vector info

clearline:
	mov al,08H              ; backspace
	call printchar          ; backspace one character
	dec cx                  ; decrease backspace count
	cmp cx,0                ; is it 0?
	jne clearline           ; if not, continue to backspace
	pop cx                  ; else restore chunk count
	dec cx                  ; decrease counter
	cmp cx,0                ; is it zero?
	je done                 ; if so, quit
	push cx                 ; and store it so we can reuse cx
	call linenumber         ; print next line number
	mov cx,CHUNKSIZE        ; prepare to print next chunk

chunkloop:       
	push ds                 ; ds needs to be set to 0 to access vector
	mov ax,0                ; area as data, then reset to = this segment
	mov ds,ax
	lodsb                   ; get next vector byte in al
	pop ds
	mov dl,al               ; save vector byte
	shr al,1                ; shift to get at high nybble
	shr al,1
	shr al,1
	shr al,1
	mov di,ax               ; prepare to index into digit list 
	mov al,byte ptr hexdgs[di]
	call printchar          ; print indexed hex digit
	mov al,dl               ; remember vector byte stored in dl
	and al,0FH              ; mask out high nybble
	mov di,ax
	mov al,byte ptr hexdgs[di]
	call printchar
	dec cx                  ; decrement counter
	jz chunksloop           ; next chunk of data 
	jmp chunkloop           ; fetch next vector byte in this chunk

done:
	mov bx, offset msg
	call print              ; inform user the fun is over
dead:
	jmp dead                ; He's dead, Jim. Loops forever.

printchar:
	push ax                 ; al has char to print
	push bx                                                         
	mov  ah, 0EH            ; int 10h function to print char
	mov  bh,0               ; active page
	int 10H
	pop bx
	pop ax
	ret

digits: db 0,0,0                     ; memory to hold line number digits
				     ; most significant to least
linenumber:                          ; print line number
	push ax                      ; save registers
	push bx
	push dx
	push cx
	mov ax, CHUNKS               ; compute line number
	sub ax,cx                    ; cx counts down from CHUNKS
	inc ax                       ; ax now has line number
	mov bx,2                     ; counter for decimal digits
cycle:  xor dx,dx                    ; prepare to divide into dx!ax
	div byte ptr base            ; divide by 10. ah has remainder
	add ah,48                    ; make into ascii digit
	mov byte ptr digits[bx],ah   ; store digit for later print
	xor ah,ah                    ; prepare for next division
	cmp bx,0                     ; last digit?
	je endcycle                  ; yes, print out digits
	dec bx                       ; no, decrease digit counter
	jmp cycle                    ; and loop to find next digit

endcycle:                           ; print decimal digits found above
	mov al,byte ptr digits[0]
	call printchar              ; hundreds digit
	mov al,byte ptr digits[1]
	call printchar              ; tens digit
	mov al,byte ptr digits[2]
	call printchar              ; ones digit
	mov al, 3AH
	call printchar              ; print a colon
	pop cx
	pop dx
	pop bx
	pop ax 
	ret

kbhit:                          ; Wait for user to strike a key
	push ax
	mov ah, 0
	int 16H
	pop ax
	ret

print:  push    ax              ; prints string, bx has buffer
	push    bx              ; and string ends with 0
	push    cx
	push    dx
pr1:    push    bx              ; push pointer
	mov     al, ds:[bx]     ; get char
	or      al, al          ; check for end
	jz      pr2             ; jump to finish
	mov     ah, 0eh         ; print code
	mov     bh,0            ; pageint     
	int     10h             ; int to BIOS
	pop     bx              ; restore pointer to bx
	inc     bx              ; next char
	jmp     pr1             ; loop
pr2:
	pop     bx
	pop     dx
	pop     cx
	pop     bx
	pop     ax
	ret        


bblock  endp

hexdgs:  db  '0123456789ABCDEF'
base:    db   0AH
prompt:  db  'Line number:Table entry -> ',0 
msg:     db   0dh,0ah,'Done. Reboot now :-)',0 
banner:  db  'BIOS Jump Table Lister, by Terry R. McConnell',0dh,0ah,0

cseg    ends

end bblock


