/* v2text.c: read a 2048 byte copy of bios jump table (+ extended) 
and print hex codes (2) for each byte to the screen just like the boot sector
code bblurt.asm. 

This should only be run under 16 bit DOS since we assume that a char is
16 bit, etc.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef unsigned char byte;

#define NN 2048      /* Number of bytes in the jump table */
#define DEF_FN "VECTOR.BIN"    /* Default file to blurt */
#define MAX_FN 32
#define MY_NAME "v2text"
#define VERSION "1.0"
#define USAGE "v2text [-2 -i -h -v] [filename]"
#define HELP "-v: give version number and exit\n" 
#define HELP1 "-h: print this helpful information\n" 
#define HELP5 "-i: interactive: wait to user to hit return in format 1\n"
#define HELP6 "-i: print line numbers in format 2\n"
#define HELP2 "-2: provide output in bblurt2.bin format\n"
#define HELP3 "filename: path to vector binary (default: vector.bin)\n"
#define HELP4 "output goes to stdout. Redirect as needed.\n"

static char hexdgs[] = "0123456789ABCDEF";
static byte vbuf[NN];
static char vfname[MAX_FN];

int main(int argc, char **argv){

        FILE *in;
        int interactive = 0;
        int debug = 0;
        int format = 1;
        int i=0,j=0,n;
        byte c;
        byte x,y,z;

        /* Default file name to open */
        strcpy(vfname,DEF_FN);

        /* Process command line */
        while(++j < argc){
                if((argv[j][0] == '-')||(argv[j][0]=='/'))
                        switch(argv[j][1]){ 
                                case '2':
                                        format = 2;
                                        continue;
                                case 'i':
                                case 'I':
                                        interactive = 1;
                                        continue;
                                case 'd':
                                case 'D':
                                        debug = 1;
                                        continue;
                                case 'v':
                                case 'V':
                                        printf("%s\n",VERSION);
                                        exit(0);
                                case '?':
                                case 'h':
                                case 'H':
                                        printf("%s\n",USAGE);
                                        printf("%s%s%s%s%s%s%s",HELP,HELP1,HELP2,
                                        HELP5,HELP6,HELP3,HELP4);
                                        exit(0);
                                default:
                                        fprintf(stderr,"%s: unkown option %s\n",
                                                MY_NAME,argv[j]);
                                        exit(1);
                        }
                else {     /* Not option, must be filename to open */
                        strcpy(vfname,argv[j]);  
                        break;
                }
        }

        if(j+1 < argc){
                        fprintf(stderr,"%s\n",USAGE);
                        exit(1);
        }
                
        in = fopen(vfname,"rb");
        if(!in){
                fprintf(stderr,"%s: could not open %s for binary read\n", 
                MY_NAME,vfname);
                return 1;
        }
        if(debug)printf("%s: opened %s for binary read. \n",MY_NAME,vfname);
        n = fread(vbuf,sizeof(byte),NN,in);
        if(debug)printf("%s: read %d bytes from %s\n",MY_NAME,n,vfname);
        if(n < NN){
                fprintf(stderr,"%s: did not read sufficiently many bytes. Exiting\n",
                MY_NAME);
                return 1;
        }
        if(format == 1){

        /* Print hex codes in two full (80x24) screens plus a partial
           screen to conform with what the boot block code bblock.bin does
        */

        for(i=0;i<960;i++){
                z = vbuf[i];
                x = (0xF0 & z)/16;     /* High nybble */
                y = (0xF & z);         /* Low nybble */
                printf("%c%c",hexdgs[x],hexdgs[y]);
                if(!interactive)if( (i+1)%40 == 0)printf("\n"); 
        }
        if(interactive)getchar();
        else printf("\n");
        for(i=960;i<1920;i++){
                z = vbuf[i];
                x = (0xF0 & z)/16;     /* High nybble */
                y = (0xF & z);         /* Low nybble */
                printf("%c%c",hexdgs[x],hexdgs[y]);
                if(!interactive)if((i+1)%40 == 0)printf("\n");

        }
        if(interactive)getchar();
        else printf("\n");
        for(i=1920;i<NN;i++){
                z = vbuf[i];
                x = (0xF0 & z)/16;     /* High nybble */
                y = (0xF & z);         /* Low nybble */
                printf("%c%c",hexdgs[x],hexdgs[y]);
                if(!interactive)if((i+1)%40 == 0)printf("\n");
        }
        if(interactive)getchar();
        }
        else { /* Format = 2 */
                for(n=0;n<NN;){
                        if(interactive)printf("%d:",n/4+1);
                        for(i=0;i<4;i++){
                                z = vbuf[n+i];
                                x = (0xF0 & z)/16;     /* High nybble */
                                y = (0xF & z);         /* Low nybble */
                                printf("%c%c",hexdgs[x],hexdgs[y]);
                        }
                        printf("\n");
                        n += 4;
                }
        }
        fclose(in);
        return 0;
}

