/*---------------------------------------
   win2min -- simple windows front end for mloader program
   displays a logo, then runs wini.exe. 
  ---------------------------------------*/


#include <windows.h>
#include <stdlib.h>

#define LOGONAME "Bigm"
#define MAX_ARGS 10
#define USAGE "Usage: win2min [/q] -- boot Minix ([without warning])"
#define WARNING "Warning: This will shut down Windows and boot Minix. Continue ?"

LRESULT CALLBACK WndProc      (HWND, UINT, WPARAM, LPARAM) ;
extern int DisplayLogo(HINSTANCE);

static char szAppName[] = "win2min" ;


int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
     {

       int quiet = 0;   /* flag: don't warn user if set */
       char *CommandLine;
       int argc = 0;
       char *argv[MAX_ARGS];
       int err_val;
       char MsgBuf[80];


       CommandLine = malloc(strlen(szCmdLine)+1);
       if(CommandLine == NULL){
       	MessageBox(NULL, "Unable to malloc Command Line Array",
         	szAppName, MB_ICONERROR | MB_OK);
        	return 1;
       }
       strcpy(CommandLine,szCmdLine);

     /* Break command line into argv, argc   */


     argv[argc++] = szAppName;
     if((argv[argc] = strtok(CommandLine," 	"))!=NULL){
        argc++;
     	  while((argv[argc] = strtok(NULL," 	"))!= NULL)argc++;
     }


     if(argc >1 )
     		if(strcmp(argv[1],"/q") != 0){
         	MessageBox(NULL, USAGE,
         	szAppName, MB_ICONERROR | MB_OK);
        		return 1;
         }
         else quiet = 1;

     DisplayLogo(hInstance);
     

     if(!quiet)
      if(  MessageBox(NULL, WARNING,
            szAppName, MB_ICONQUESTION | MB_OKCANCEL) == IDCANCEL)
      	return 0;


     /* Goodbye Windows 95 !!! */

     err_val = ShellExecute(NULL,"open","minix.pif",NULL,NULL,0);
     if(err_val < 32){
      wsprintf( MsgBuf,"Error executing minix.pif. Error Value = %d",err_val);
     	MessageBox(NULL, MsgBuf,
         	szAppName, MB_ICONERROR | MB_OK);
      return 1;
     }
     return 0 ;
     }


void DrawBitmap(HDC hdc, HBITMAP hBitmap, int xStart, int yStart);

   /* Display  Logo. 
      This rudely displays to the middle of the desktop screen
      while locking all updates -- idea from Petzold's scramble.c,
      p. 195 ff of THE BOOK */

     int DisplayLogo(  HINSTANCE hInstance)
     {

         HDC hdc;
         HBITMAP mybmhandle;
         HBITMAP savedbmhandle;
         HDC hdcMem;
         POINT Size, Origin, Screen;
         BITMAP bm;
         


         /* Load the bitmap resource */
         mybmhandle = LoadBitmap(hInstance,LOGONAME);
         GetObject(mybmhandle, sizeof (BITMAP), (LPVOID) &bm);

         /* Figure out size of image so we can make the window
            of the same size */
        Size.x = bm.bmWidth;
        Size.y = bm.bmHeight;

        /* Get the horizontal and vertical sizes of the screen */
        Screen.x = GetSystemMetrics(SM_CXSCREEN);
        Screen.y = GetSystemMetrics(SM_CYSCREEN);

        Origin.x =  (Screen.x -Size.x)/2 ;
        Origin.y = (Screen.y - Size.y)/2;

        /* Create window in which to draw the logo so that it
           it is of the proper style, size, and centered on the
           screen. */



         if(LockWindowUpdate(GetDesktopWindow()))
           {

           hdc = CreateDC("DISPLAY",NULL,NULL,NULL);

           /* Copy the part of the screen that will display the
               logo to a safe place so it can be restored */
           savedbmhandle = CreateCompatibleBitmap(hdc, Size.x, Size.y);
           hdcMem = CreateCompatibleDC(hdc);
           SelectObject(hdcMem, savedbmhandle);
           BitBlt(hdcMem,0,0,Size.x, Size.y, hdc,(Screen.x -Size.x)/2,
            (Screen.y - Size.y)/2,SRCCOPY);

         /* Display the logo bitmap */
           DrawBitmap(hdc, mybmhandle,(Screen.x - Size.x)/2
           	,(Screen.y - Size.y)/2);


           Sleep(3000);

         /* Restore the screen to what it was before we displayed
            the logo */

            BitBlt(hdc, (Screen.x - Size.x)/2, (Screen.y - Size.y)/2,
            	Size.x, Size.y, hdcMem, 0, 0, SRCCOPY);

         /* Clean up */
           DeleteObject((HGDIOBJ)mybmhandle);
           DeleteObject((HGDIOBJ)savedbmhandle);
           DeleteDC(hdcMem);
           ReleaseDC("DISPLAY",hdc);

           LockWindowUpdate(NULL);
         }
         return TRUE;
     }

     /* Display Bit Map routine, taken from Petzold p. 190 */

     void DrawBitmap(HDC hdc, HBITMAP hBitmap, int xStart, int yStart)
     {
     	BITMAP bm;
      HDC hdcMem;
      DWORD dwSize;
      POINT ptSize, ptOrg;

      hdcMem = CreateCompatibleDC(hdc);
      SelectObject(hdcMem, hBitmap);
      SetMapMode(hdcMem, GetMapMode(hdc));

      GetObject(hBitmap, sizeof (BITMAP), (LPVOID) &bm);
      ptSize.x = bm.bmWidth;
      ptSize.y = bm.bmHeight;
      DPtoLP(hdc,&ptSize,1);

      ptOrg.x = 0;
      ptOrg.y = 0;
      DPtoLP(hdcMem,&ptOrg,1);

      BitBlt(hdc,xStart, yStart,ptSize.x, ptSize.y,hdcMem,
       			ptOrg.x, ptOrg.y, SRCCOPY);

      DeleteDC(hdcMem);
      return;
   }

