/* wini.c: code to boot a harddrive partition or floppy drive from DOS.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   
 */


/*===========================================================================*

  Prog    : WINI
	    Permite bootear de una particion. Pensado para boot desde
	    menu de DOS para Minix (ShoeLace).

  Compile : make wini (Using Borland C/C++ 4.5)          

  uso     : en config.sys
	    shell=wini.exe -f n -h n -w <drive:directory> nro-part 
	    
	    -h       drive a bootear (0 o 1)
	    -f       Boot from floppy. (1 = a:, 2 = b:)
	    -w       working directory
	    nro-part particion donde bootear
	    -d       debug flag
	    -v       skip the vm86 check altogether

	  : At Dos command line
	    [path]wini[.exe] [-f n -h n -w <drive:directory> [-v] nro-part]

/*===========================================================================*

	Funcionamiento del programa
	
	a) lee informacion del Hard Disk (bootblock) y detecta
	   los tipos de particiones que tiene.
	   
	b) muestra las particiones
	
	c) si se paso como linea de comando un nro de particion
	   es booteada. Si no es valida lo indica y finaliza.

	d) Program switches to real mode if it detects the presence of
	   vm86 mode with VCPI interface. (This would be the case,
	   e.g, if extended memory blocks are being provided by EMM386.EXE or
	   compatible EMS manager.) This functionality is implemented in
	   the file vm86.asm.

/*===========================================================================*/
/*

   V1.2 - Support for booting from Virtual 86 mode with Virtual Control
	  Program Interface (VCPI). Support for booting from a 1.44M floppy.
	  (By Terry R. McConnell -- tmc@barnyard.syr.edu -- 8/97)

   V1.1 - deteccion que no bootea de 2do disco. Cuando se llama al boot
	  block en 0:7c00 incorrectamente se pasaba HD1 como parametro sin
	  sacarlo de la variable dr en main. Se corrigio y boot ok del 
	  2do disco. (13/4/95)  C. Tantignone 

   V1.0 - primer release del programa, tras muchos beta test.
*/


#define  VERSION "1.2"         /* version */
#define USAGE "[path]wini[.exe] [-m -f n -h n -w <drive:directory> [-v] nro-part]"

#include <dos.h>
#include <bios.h>
#include <alloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dir.h>
#include <mem.h>
#include <conio.h>   
#include <fcntl.h>
#include <sys\stat.h>

/*===========================================================================*
/* define's */
/*===========================================================================*/

#define RESETDSK        0       /* function reset disk       */
#define GETSTATUS       1       /* function get drive status */
#define READVBLK        2       /* function read block       */
#define WRITEVBLK       3       /* function write disk block */
#define VERIFYTRK       4       /* function verify disk addr */
#define BLOCK           512     /* buffer size for block     */
#define HD1             0x80    /* hard disk #1              */
#define HD2             0x81    /* hard disk #2              */
#define HD3             0x82
#define FDA             0x00    /* floppy disk a:            */
#define FDB             0x01    /* floppy disk b:            */
#define HDBOOT(d)       disk_io(READVBLK, d, 0, 0, 1, mbr)
#define RDBOOT(d,t,h,s) disk_io(READVBLK, d, t, h, s, buffer)
#define TABLEOFFSET     0x1be   /* offset in boot sector     */
#define VECLEN          2048    /* vector table size         */
#define VECTOR_FILE     "VECTOR.BIN"  /* vector image file   */

/*===========================================================================*
 * Description of entry in partition table
/*===========================================================================*/

struct part_entry {
	unsigned char   bootind; /* boot indicator 0/0x80       */
	unsigned char   start_head;     /* head value for first sector  */
	unsigned char   start_sec;      /* sector value for first sector*/
	unsigned char   start_cyl;      /* track value for first sector */
	unsigned char   sysind;         /* system indicator 00=?? 01=DOS*/
	unsigned char   last_head;      /* head value for last sector   */
	unsigned char   last_sec;       /* sector value for last sector */
	unsigned char   last_cyl;       /* track value for last sector  */
	long    lowsec;                 /* logical first sector         */
	long    size;                   /* size of partion in sectors   */
};

/*===========================================================================*
/* variables */
/*===========================================================================*/

static unsigned char buffer[BLOCK];      /* general buffer for drive */
static unsigned char mbr[BLOCK];         /* master buffer for drive */
static int part;                         /* partition to be booted */
static int use_floppy;                   /* flag to boot from floppy */
static int h_option;
/* Dont declare the following static: it is referenced in vm86.asm */
int debug = 0;                        /* debug flag */
static unsigned char vecbuf[VECLEN];     /* vector buffer */

extern int check_VCPI_present(void);   /* Implemented in vm86.asm */
extern void exit_vm86(void);           /* ditto */

/* Forward declarations */
void dpl_entry(int,struct part_entry *);
void dpl_partitions(void);
void prterr(int);
char *systype(int);


/*===========================================================================*
 *                              disk_io                                      * 
 *===========================================================================*/
/* realizo funciones de I/O sobre disco */
int disk_io(int func, int disk, int track, int head, int sector, char *buf)    
{
   int byt, fnc,i;
   char *access;        /* pointer to function type */
   char *ptr;
   
   fnc = func;       /* copy function code */
   ptr = buf;        /* copy buffer pointer */

   switch (fnc) {
	 case GETSTATUS : access = "Status";  break;
	 case READVBLK  : access = "Read";    break;
	 case WRITEVBLK : access = "Write";   break;
	 case VERIFYTRK : access = "Verify";  break;
	 default :        access = "Unknown"; break;
	 }

   /* leo o escribo bloque del diskette */

   if (fnc == READVBLK) {
      if (debug)
	 printf("wini: cleaning buffer ...\n");

      for (i = 0; i < BLOCK; i++)
	 *ptr++ = '\0';
      }         

   if (debug)
      fprintf(stderr, "wini: %s access disk %d, head %d, track %d, sector %d\n", access,
	  disk, head, track, sector);

   byt = biosdisk(fnc, disk, head, track, sector, 1, buf);

   if (byt != 0)  /* trato de hacerlo 2 veces si hay error */
      byt = biosdisk(fnc, disk, head, track, sector, 1, buf);

   /* si hubo error imprimo mensaje en pantalla */

   if (byt != 0) {
      fprintf(stderr, "\nwini: %s error at disk %d (track %d, head %d, sector %d). ",(fnc == 2)? "Read": "Write",
			     disk, track, head, sector);
      prterr(byt);     /* print error */
      }
   return(byt);
}

/*===========================================================================*
 *                              prterr                                       * 
 *===========================================================================*/
void prterr(int byt)       /* print biosdisk error */
{
      fprintf(stderr,"wini: ");

      if ((byt & 0x80) != 0)
	 fprintf(stderr, "Disk timed-out.\n");
      else if ((byt & 0x40) != 0)
	 fprintf(stderr, "Seek failure.\n");
      else if ((byt & 0x20) != 0)
	 fprintf(stderr, "Controller error.\n");
      else if ((byt & 0x10) != 0)
	 fprintf(stderr, "Data error on disk read (CRC).\n");
      else if ((byt & 0x08) != 0)
	 fprintf(stderr, "DMA overrun on operation.\n");
      else if ((byt & 0x04) != 0)
	 fprintf(stderr, "Request sector not found.\n");
      else if ((byt & 0x02) != 0)
	 fprintf(stderr, "Disk write protected.\n");
      else if ((byt & 0x01) != 0)
	 fprintf(stderr, "Ilegal command passed to driver.\n");
      else printf("Status = 0x%x \n", byt);

   return;
}

/*==========================================================================*
 *                              systype                                     *
 *==========================================================================*/
char *systype(type)
int type;
{
/* Convert system indicator into system name. */

  switch(type) {
	case 0:              return("  None ");
	case 1:              return("DOS-12 ");
	case 2:              return(" XENIX ");
	case 3:              return("XNX-OLD");
	case 4:              return("DOS-16 ");
	case 5:              return("DOS-EXT");
	case 6:              return("DOS-BIG");
	case 8:              return("  AIX  ");
	case 10:             return("  OPUS ");
	case 0x51:           return("NOVELL ");
	case 0x52:           return("  CPM  ");
	case 0x63:           return("386/IX ");
	case 0x64:           return("NOVELL ");
	case 0x75:           return(" PCIX  ");
	case 0x81:           return(" MINIX ");
	case 0x82:           return("LinuxSW");
	case 0x83:           return("LinuxEX");
	case 0xDB:           return("  CPM  ");
	case 0xFF:           return("BADBLKS");
	default:             return("Unknown");
  }
}

/*===========================================================================*
 *                              main                                         * 
 *===========================================================================*/
void main(int argc, char **argv)
{
    unsigned myseg, off, segb, offb, segv, offv;
    char oem[10];
    int low_cyl, low_sec, low_head;
    struct part_entry *pe;
    FILE *fd;
    unsigned char dr; /* drive */
    char path[80];    /* path de trabajo */
    int i,j;          /* temp variable */
    int check_vm86=1;
    int boot_mbr=0;
    
    part = 0;         /* invalid partition to be booted */
    path[0] = '\0';   /* init path string */
    dr = HD1;         /* primer hard disk */    


    printf("\nDisk Booter V%s - Free Software GPL\n", VERSION);
    printf("1995 - C.M.T. Software Utilities Inc.\n");
    printf("1.1-1.2 revision by Terry R. McConnell\n\n");

    j = 1;     /* points to first variable */
    if (argc > 1) {      /* must check each parameter */
       while (argv[j][0] == '-') {
	  switch (argv[j][1]) {             /* en funcion del flag         */
	     case 'd':               /* debug flag */
	     case 'D':
		       debug = 1;
		       printf("wini: debug flag on ...\n");
		       break;
	     case 'm':
	     case 'M':
			boot_mbr = 1;
			break;

	     case 'h':               /* hard drive flag */
	     case 'H':
		       j++;   /* points to new argument */
		       h_option = 1;
		       if(use_floppy == 1){
			  printf("wini: Cannot use both -f and -h options!\n");
			  exit(1);
		       }
		       if (argv[j][0] == '1')
			  dr = HD1;
		       else if (argv[j][0] == '2')
			  dr = HD2;
		       else if (argv[j][0] == '3')
			  dr = HD3;
		       else {
			  printf("wini: invalid drive %s.\n", argv[j]);
			  exit(1);
			  }
		       if (debug) {
			  printf("wini: hard drive %c selected.\n",(dr==HD1)?'C':'D');
			  }
		       break;
		case 'F':
		case 'f':
		       use_floppy = 1;
		       j++;   /* points to new argument */
		       if(h_option == 1){
			  printf("wini: Cannot use both -f and -h options!\n");
			  exit(1);
		       }
		       if (argv[j][0] == '1')
			  dr = FDA;
		       else if (argv[j][0] == '2')
			  dr = FDB;
		       else {
			  printf("wini: invalid floppy drive %s.\n", argv[j]);
			  exit(1);
			  }
		       if (debug) {
			  printf("wini: floppy drive %c selected.\n",(dr==FDA)?'A':'B');
			  }
		       break;

	     case 'v':  /* skip vm86 test */
			check_vm86=0;
			break;
	     case 'w':               /* working path */
	     case 'W':
		       j++;   /* points to new argument */
		       strcpy(path,argv[j]);  /* copy new path */
		       if (debug) {
			  printf("wini: working path is \"%s\".\n", path);
			  }
		       strcat(path,"\\");  /* dir separator */    
		       break;
	     case '?':
		       printf("%s\n",USAGE);
		       exit(0);

	     default:
		       printf("wini: invalid option \"%s\".\n", argv[j]);
		       printf("%s\n",USAGE);
		       exit(1);
		       break;
	     }
	  j++;   /* new pointer */
	  }
       if (argc == (j+1)) {
	  part = atoi(argv[j]); /* convert part number */
	  if ((part<=0)||(part>4)) {
	     fprintf(stderr,"wini: invalid partition %s.\n", argv[j]);
	     part=0;
	     }
	  }
       }

  if(use_floppy == 0){

    printf("\nwini: Information for Hard Drive %c ...\n", (dr==HD1)?'C':'D'); 
    

    if (debug)
       printf("wini: reading master bootblock on hard disk ...\n");

    if (HDBOOT((unsigned int)dr)) {
       printf("wini: error reading master boot block from hard drive %d ...\n", (dr==HD1)?1:2);
       exit(1);
       }

    if (debug)
       printf("wini: checking master partition table info ...\n");

    if (mbr[510] != 0x55 || mbr[511] != 0xAA) {
       printf("wini: invalid master partition table.\n",i);
       exit(1);
       }

    dpl_partitions();    /* display partitions */
     
    if ((part == 0) && (boot_mbr == 0))
       exit(0);
	
    if (debug)
       printf("\nwini: checking partition %d info ...\n", part);

    i = part;       /* points to partition */
    pe = (struct part_entry *)&mbr[TABLEOFFSET];
    while (--i > 0) pe++;  /* finds partition */
    
    if (pe->sysind == 0) {  /* avoid null partitions */
       printf("wini: null partition. Exiting.\n");
       exit(1);
       }

    low_cyl = (pe->start_cyl & 0xff) + ((pe->start_sec & 0xc0) << 2);
    low_sec = pe->start_sec & 0x3f;
    low_head = pe->start_head;

    if (RDBOOT((unsigned int)dr, low_cyl, low_head, low_sec)) {
       printf("wini: error reading boot block ...\n");
       exit(1);
       }

    if (debug)
       printf("\nwini: checking partition %d signature ...\n", part);

    if (buffer[510] != 0x55 || buffer[511] != 0xAA) {
       printf("wini: invalid partition signature.\n",i);
       exit(1);
       }

    strncpy(oem,&buffer[3],8);
    oem[8] = '\0';

    printf("\nPartition %d - OEM : %s\n",part, oem);
  }
  else {    /* use_floppy == 1 */

    printf("\nwini: Information for Floppy Drive %c ...\n", (dr==FDA)?'A':'B'); 
    

    if (debug)
       printf("wini: reading bootblock on floppy disk ...\n");

    if (RDBOOT((unsigned int)dr,0,0,1)) {
       printf("wini: error reading master boot block from floppy drive %d ...\n", (dr==FDA)?1:2);
       exit(1);
       }

    
  }

    strcat(path, VECTOR_FILE);    /* encadeno ambos paths */

    if (debug)
       printf("\nwini: loading vector area into core (%s) ...\n", path);

    if ((fd = fopen(path,"rb")) == NULL) {
       perror("fopen vector file");
       exit(errno);
       }

    if (fread(vecbuf,sizeof(char),VECLEN,fd) != VECLEN) {
       perror("fread vector file");
       exit(errno);
       }

    (void)fclose(fd);       /* close vector file */

    if (debug) {
       myseg = peek(0,(0x13*4)+2);
       off = peek(0,0x13*4);
       printf("\nwini: int 13h points to %x:%x .\n", myseg,off);
       }

    if(boot_mbr) {
	segb = FP_SEG(mbr);
	offb = FP_OFF(mbr);
    }
    else {
	segb = FP_SEG(buffer);    /* get segment from buffer */
	offb = FP_OFF(buffer);    /* get offset */
    }
    segv = FP_SEG(vecbuf);    /* far pointer to vector buffer */
    offv = FP_OFF(vecbuf);    /* get offset */

    if (debug) {
       printf("\nwini: buffer address from %x:%x ...\n", segb, offb);
       printf("\nwini: vector address from %x:%x ...\n", segv, offv);
       }

    if (debug)
       printf("\nwini: jumping to boot code ...\n");

/* Continuing beyond here requires that we be in real-mode, since the
   kernel cannot ultimately be booted in vm86 mode. Accordingly, we check
   here for the presence of vm86 mode and go to real mode if so. */

/* we provide option to skip this using -v flag */
    if(check_vm86)
    if(check_VCPI_present()){
	if(debug)
		printf("\nwini: VCPI services present. Going to real mode.\n");
	exit_vm86();         /* If necessary, go to real mode */
	if(debug)
		printf("\nwini: successful return from exit_vm86\n");
    }


    /* copy boot code to 0:7c00 */
    asm push    ds
    asm push    es
    asm push    di
    asm push    si
    asm cld
    asm mov     ax, segb         /* segment to buffer */
    asm mov     ds, ax
    asm mov     si, offb         /* offset to buffer */
    asm mov     ax, 7c0h        /* segment to destination */
    asm mov     es, ax
    asm mov     di, 0
    asm mov     cx, BLOCK       /* size */
    asm rep movsb
    asm pop     si
    asm pop     di
    asm pop     es
    asm pop     ds

    /* copy code from vector buffer to vector area */

    asm push    ds
    asm push    es
    asm push    di
    asm push    si
    asm cld
    asm mov     ax, segv      /* loads ds (source segment) */
    asm mov     ds, ax
    asm mov     si, offv      /* source offset */
    asm mov     ax, 0
    asm mov     es, ax       /* load es (destination segment) */
    asm mov     di, 0        /* destination offset */
    asm mov     cx, VECLEN   /* vector area length */
    asm rep movsb
    asm pop     si
    asm pop     di
    asm pop     es
    asm pop     ds

    asm sti

    /* get pointers to partition area */
    
    myseg = FP_SEG(pe);        /* far pointer to vector buffer */
    off = FP_OFF(pe);        /* get offset */

    asm mov     ax, myseg      /* segment to es */
    asm mov     es, ax
    asm mov     si, off      /* offset to si */
    /* V1.1 modificacion se pasa variable dr en dl con disco de boot */
    asm mov     dl, dr       /* drive number */


    /* jmp 0:7c00h to code */
    asm db      0eah,00,07ch,0,0

}

/*===========================================================================*
/* dpl_partition : display partition info from structure
/*===========================================================================*/
void dpl_partitions()
{
	struct  part_entry      *pe;
	int     i;

	printf("\nPartition      Type        SysId     Begin      End     Act  Offset     Size\n");
	pe = (struct part_entry *)&mbr[TABLEOFFSET];
	for (i = 1 ; i <= 4 ; i++) {
		dpl_entry(i,pe);
		pe++;
	}
}

/*===========================================================================*
/* dpl_entry : display an entry 
/*===========================================================================*/
void dpl_entry(number ,entry)
int number;
struct  part_entry      *entry;
{
	int     low_cyl,high_cyl,temp,low_sec,high_sec,low_head,high_head;
	char    *typestring;
	char    active;
	char    begstr[80];
	char    highstr[80];

	if (entry->sysind == 0)
	   return;

	typestring = systype(entry->sysind);

	printf("%5d         %s       %3d ",number,typestring,entry->sysind);
	temp = entry->start_sec & 0xc0;
	low_cyl = (entry->start_cyl & 0xff) + (temp << 2);
	low_sec = entry->start_sec & 0x3f;
	low_head = entry->start_head;
	temp = entry->last_sec & 0xc0;
	high_cyl = (entry->last_cyl & 0xff) + (temp << 2);
	high_sec = entry->last_sec & 0x3f;
	high_head = entry->last_head;
	sprintf(begstr,"%d/%d/%d",low_cyl,low_sec,low_head);
	sprintf(highstr,"%d/%d/%d",high_cyl,high_sec,high_head);
	printf("%10s  %10s", begstr,highstr);
	if ((entry->bootind & 0xff) == 0x80)
		active = 'A';
	else
		active = 'N';
	printf("   %c %8ld %8ldKb\n",active, entry->lowsec, entry->size/2);
	return;
}
