/*===========================================================================*

  Prog    : WRBOOT
            escribe boot block en floppy disk.

  Compile : bcc -ml -O -Z wini.c          

  uso     : wrboot
  

/*===========================================================================*

	Funcionamiento del programa
	
	a) lee archivo bb.com, hace pad a 512 bytes y escribe boot
           block en floppy disk
           
/*===========================================================================*/

#define  VERSION "1.0"         /* version */

#include <dos.h>
#include <bios.h>
#include <alloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dir.h>
#include <mem.h>
#include <conio.h>   
#include <fcntl.h>
#include <sys\stat.h>

/*===========================================================================*
/* define's */
/*===========================================================================*/

#define RESETDSK        0        /* function reset disk       */
#define GETSTATUS       1        /* function get drive status */
#define READVBLK        2        /* function read block       */
#define WRITEVBLK       3        /* function write disk block */
#define VERIFYTRK       4        /* function verify disk addr */
#define BLOCK           512      /* buffer size for block     */
#define BOOTBLOCK       "BBLOCK.BIN" /* boot code                 */

/*===========================================================================*
/* variables */
/*===========================================================================*/

static unsigned char buffer[BLOCK];      /* general buffer for drive */
static int debug=0;                      /* debug flag */

/*===========================================================================*
 *                              prterr                                       * 
 *===========================================================================*/
prterr(int byt)       /* print biosdisk error */
{
      fprintf(stderr,"\nwrboot: ");
      
      if ((byt & 0x80) != 0)
	 fprintf(stderr, "Disk timed-out.\n");
      else if ((byt & 0x40) != 0)
	 fprintf(stderr, "Seek failure.\n");
      else if ((byt & 0x20) != 0)
	 fprintf(stderr, "Controller error.\n");
      else if ((byt & 0x10) != 0)
	 fprintf(stderr, "Data error on disk read (CRC).\n");
      else if ((byt & 0x08) != 0)
	 fprintf(stderr, "DMA overrun on operation.\n");
      else if ((byt & 0x04) != 0)
	 fprintf(stderr, "Request sector not found.\n");
      else if ((byt & 0x02) != 0)
	 fprintf(stderr, "Disk write protected.\n");
      else if ((byt & 0x01) != 0)
	 fprintf(stderr, "Ilegal command passed to driver.\n");
      else printf("Status = 0x%x \n", byt);

   return(0);
}

/*===========================================================================*
 *                              main                                         * 
 *===========================================================================*/
void main()
{
    FILE *fd;
    int byt;
    

    printf("\nFloppy Boot Disk Writer V%s - All rights reserved\n", VERSION);
    printf("1995 - C.M.T. Software Utilities Inc.\n");

    printf("wrboot: reading boot block image ...\n");

    if ((fd = fopen(BOOTBLOCK,"rb")) == NULL) {
       perror("fopen boot file");
       exit(errno);
       }

    if (fread(buffer,sizeof(char),BLOCK,fd) == 0) {
       perror("fread boot file");
       exit(errno);
       }

    (void)fclose(fd);       /* close boot file */

    printf("wrboot: marking DOS style boot ...\n");
    
    buffer[510] = 0x55;
    buffer[511] = 0xAA;

    printf("wrboot: writting boot into floppy A drive ...\n");

    byt = biosdisk(WRITEVBLK, 0, 0, 0, 1, 1, buffer);

    if (byt != 0)  /* trato de hacerlo 2 veces si hay error */
       byt = biosdisk(WRITEVBLK, 0, 0, 0, 1, 1, buffer);

   /* si hubo error imprimo mensaje en pantalla */

   if (byt != 0) {
      prterr(byt);     /* print error */
      printf("\nwrboot: write error to floppy, please retry ...\n");
      exit(1);
      }

   printf("wrboot: write done succesfully ...\n");

}
